using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using ProjectBoat_bae.Entities;
using ProjectBoat_bae.MovementStrategy;
using ProjectBoat_bae.DrawningObjects;

namespace ProjectBoat_bae
{
    public partial class FormBoat : Form
    {
        // -   

        private Drawningboat? _drawingBoat;
        private AbstractStrategy? _abstractStrategy;
        ///  
        public Drawningboat? SelectedBoat { get; private set; }

        public FormBoat()
        {
            InitializeComponent();
            _abstractStrategy = null;
            SelectedBoat = null;
        }

        //   
        private void Draw()
        {
            if (_drawingBoat == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxBoat.Width,
            pictureBoxBoat.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawingBoat.DrawTransport(gr);
            pictureBoxBoat.Image = bmp;
        }

        //   
        private void buttonMove_Click(object sender, EventArgs e)
        {
            if (_drawingBoat == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "button_top":
                    _drawingBoat.MoveTransport(DiretionType.Up);
                    break;
                case "button_bottom":
                    _drawingBoat.MoveTransport(DiretionType.Down);
                    break;
                case "button_left":
                    _drawingBoat.MoveTransport(DiretionType.Left);
                    break;
                case "button_right":
                    _drawingBoat.MoveTransport(DiretionType.Right);
                    break;
            }
            Draw();
        }
        private void buttonClickMotorBoat_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }

            Color dopColor = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialogAddColor = new();
            if (dialogAddColor.ShowDialog() == DialogResult.OK)
            {
                dopColor = dialogAddColor.Color;
            }

            _drawingBoat = new DrawningMotorBoat(random.Next(100, 300),
            random.Next(1000, 3000), color, dopColor, Convert.ToBoolean(random.Next(0, 2)),
            Convert.ToBoolean(random.Next(0, 2)),
            pictureBoxBoat.Width, pictureBoxBoat.Height);
            _drawingBoat.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }
        private void buttonClick_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }
            _drawingBoat = new Drawningboat(random.Next(100, 300), random.Next(1000, 3000), color, pictureBoxBoat.Width, pictureBoxBoat.Height);
            _drawingBoat.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        private void buttonStep_Click(object sender, EventArgs e)
        {
            if (_drawingBoat == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _abstractStrategy = comboBoxStrategy.SelectedIndex
                switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(new DrawningObjectBoat(_drawingBoat), pictureBoxBoat.Width, pictureBoxBoat.Height);
                comboBoxStrategy.Enabled = false;
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _abstractStrategy = null;
            }
        }

        private void buttonSelectBoat_Click_Click(object sender, EventArgs e)
        {
            SelectedBoat = _drawingBoat;
            DialogResult = DialogResult.OK;
        }
    }
}