﻿using Microsoft.Extensions.Logging;

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using ProjectBoat_bae.DrawningObjects;
using ProjectBoat_bae.Generics;
using ProjectBoat_bae.Exceptions;
using ProjectBoat_bae.MovementStrategy;
using NLog;

namespace ProjectBoat_bae
{
    public partial class FormBoatCollection : Form
    {
        // Набор объектов
        private readonly BoatsGenericStorage _storage;

        // Логер
        private readonly 
            Microsoft.Extensions.Logging.ILogger? _logger;

        public FormBoatCollection(ILogger<FormBoatCollection> logger)
        {
            InitializeComponent();
            _storage = new BoatsGenericStorage(pictureBoxCollection.Width, pictureBoxCollection.Height);
            _logger = logger;
        }

        // Заполнение listBoxObjects
        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;
            listBoxStorages.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxStorages.Items.Add(_storage.Keys[i]);
            }
            if (listBoxStorages.Items.Count > 0 && (index == -1 || index
            >= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }
            else if (listBoxStorages.Items.Count > 0 && index > -1 &&
            index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
            }
        }

        // Добавление набора в коллекцию
        private void ButtonAddObject_Click_1(object sender, EventArgs e)
        {

            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();
            _logger.LogInformation($"Added set: {textBoxStorageName.Text}");
        }

        // Удаление набора
        private void ButtonDelObject_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }

            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не всё заполнено", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogWarning("Неудачная попытка. Коллекция не добавлена, не все данные заполнены");
                return;
            }
            _storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();

            _logger.LogInformation($"Добавлен набор: {textBoxStorageName.Text}");
        }

        // Добавление объекта в набор
        private void ButtonAddBoat_Click(object sender, EventArgs e)
        {

            FormBoatConfig formPlaneConfig = new FormBoatConfig();
            formPlaneConfig.AddEvent(AddBoat);
            formPlaneConfig.Show();
        }

        private void AddBoat(Drawningboat boat)
        {
            boat._pictureWidth = pictureBoxCollection.Width;
            boat._pictureHeight = pictureBoxCollection.Height;
            if (listBoxStorages.SelectedIndex == -1) return;
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null) return;

            if (obj + boat)
            {
                MessageBox.Show("Объект добавлен");
                pictureBoxCollection.Image = obj.ShowBoats();
                _logger.LogInformation($"Добавлен объект {obj}");
            }
            else
            {
                MessageBox.Show("Не удалось добавить объект");
            }

        }

        //удаление объекта
        private void ButtonRemoveBoat_Click(object sender, EventArgs e)
        {
           
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удалить", MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            try
            {
                int pos = Convert.ToInt32(textBoxBoat.Text);
                if (obj - pos != null)
                {
                    MessageBox.Show("Объект удален");
                    pictureBoxCollection.Image = obj.ShowBoats();
                    _logger.LogInformation($"лодка удалена из набора {listBoxStorages.SelectedItem.ToString()}");
                }
                else
                {
                    MessageBox.Show("Объект не удален");
                    _logger.LogWarning($"лодка не удалена из набора {listBoxStorages.SelectedItem.ToString()}");
                }
            }
            catch (BoatNotFoundException ex)
            {
                MessageBox.Show(ex.Message);
                _logger.LogWarning($"BoatNotFound: {ex.Message} in set {listBoxStorages.SelectedItem.ToString()}");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Объекта нет");
                _logger.LogWarning("Not input");
            }
        }

        // Обновление
        private void ButtonRefreshCollection_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCollection.Image = obj.ShowBoats();
        }

        //Выбор набора
        private void listBoxStorages_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCollection.Image =
            _storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowBoats();
        }

        //сохранение
        private void SaveToolStripMenu_Click(object sender, EventArgs e)
        {
            
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogWarning($"Файл {saveFileDialog.FileName} успешно сохранен");
                }
                catch (Exception ex)
                {
                    _logger.LogWarning("Не удалось сохранить");
                    MessageBox.Show($"Не сохранилось: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);

                }
            }
        }

        //загрузка
        private void LoadToolStripMenu_Click(object sender, EventArgs args)
        {

            
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.LoadData(openFileDialog.FileName);
                    _logger.LogInformation($"Данные загружены из файла {openFileDialog.FileName}");
                    ReloadObjects();
                }
                catch (Exception ex)
                {
                    MessageBox.Show($"Не загрузилось: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogWarning($"Не удалось загрузить информацию из файла: {ex.Message}");
                }
            }
        }
    }
}
